/**
 * File: DivisionManager.java
 * --------------------------------------------------------------
 * Superclass for division manager agents.
 * 
 * Author:  Iris Lorscheid
 * Date: May, 1st 2014 
*/

package agents;

import java.io.FileWriter;
import java.io.IOException;
import java.util.Random;


/**
 * Constructor of DivisionManager
 *
 */
public class DivisionManager {
	protected double  prodValue, profit, reductProd, compensation, repProd, incrRep, pMin, pMax;
	protected int resourcesAllocated, optCapacity, maxCapacity, prodScaleRep; 
	protected Random generator;
	protected FileWriter verificationOutput;
	
	public DivisionManager(	double reductProd, int optCapacity, 
													int maxCapacity, double pMin, 
													int prodScaleRep, double incrRep,
													FileWriter verificationOutput){
							
		this.reductProd = reductProd;
		this.optCapacity = optCapacity;
		this.maxCapacity = maxCapacity;
		this.pMin = pMin;
		this.prodScaleRep = prodScaleRep;
		this.incrRep = incrRep;
		this.pMax = pMin + (prodScaleRep-1)*incrRep;
		this.verificationOutput = verificationOutput;
		this.profit = 0.0;
		this.compensation = 0.0;
		this.generator = new Random();
		this.repProd = 0.0;
	}
	
	/**
	 * Division manager receives and processes compensation payment.
	 * @param payment
	 */
	public void receiveCompensation(double payment){
		compensation = payment;
		/**
		 * Process received compensation payment.
		 * (update knowledge base)
		 * 
		 */
	}
	
	/**
	 * Divison manager decides about productivity value (decision process). 
	 * @return Productivity value
	 * @throws IOException 
	 */
	public double reportProductivity(double realP) throws IOException{
		/**
		 * Implement decision process
		 */
		return repProd;
	}
	
	/**
	 * Division manager implements the resources and calculates resulting profit.
	 * @param Pesource units allocated
	 * @return Profit value
	 */
	public double produce(int resources){	
		/**
		 *  Implement production procedure
		 * 
		 */
		return profit;
	}
	
	/**
	 * Prints the setting for the given simulation run.
	 * (Debugging method)
	 */
	public void printSetting(){
		System.out.println(new String(
				"\noptCapacity " + optCapacity+
				"\nmaxCapacity " +maxCapacity+
				"\nreductProd " + reductProd+
				"\npMin " + pMin+
				"\npMax " + pMax
				));		
	}
	
	public double getProdValue() {
		return prodValue;
	}

	public void setProdValue(double prodValue) {
		this.prodValue = prodValue;
	}
	
	public double getProfit() {
		return profit;
	}
	public void setProfit(double profit) {
		this.profit = profit;
	}
	public double getReductProd() {
		return reductProd;
	}
	public void setReductProd(double reductProd) {
		this.reductProd = reductProd;
	}
	public double getCompensation() {
		return compensation;
	}
	public void setCompensation(double compensation) {
		this.compensation = compensation;
	}
	public double getRepProd() {
		return repProd;
	}
	public void setRepProd(double repProd) {
		this.repProd = repProd;
	}
	public int getResourcesAllocated() {
		return resourcesAllocated;
	}
	public void setResourcesAllocated(int resourcesAllocated) {
		this.resourcesAllocated = resourcesAllocated;
	}
	public int getOptCapacity() {
		return optCapacity;
	}
	public void setOptCapacity(int optCapacity) {
		this.optCapacity = optCapacity;
	}
	public int getMaxCapacity() {
		return maxCapacity;
	}
	public void setMaxCapacity(int maxCapacity) {
		this.maxCapacity = maxCapacity;
	}

	public FileWriter getVerificationOutput() {
		return verificationOutput;
	}

	public void setVerificationOutput(FileWriter verificationOutput) {
		this.verificationOutput = verificationOutput;
	}
	public double getIncrRep() {
		return incrRep;
	}

	public void setIncrRep(double incrRep) {
		this.incrRep = incrRep;
	}

	public int getProdScaleRep() {
		return prodScaleRep;
	}

	public void setProdScaleRep(int prodScaleRep) {
		this.prodScaleRep = prodScaleRep;
	}


	
	
}
