/**
File: DivisonManagerLM3.java
--------------------------------------------------------
Creates a division manager object for learning model 3. 
* 
* Author:  Iris Lorscheid
* Date: May, 1st 2014 
* 
*/

package agents;

import java.io.FileWriter;
import java.io.IOException;

import knowledgebase.Beliefs;
import knowledgebase.Knowledgebase;
import learning.FicticiousPlay;
import learning.LearningAlgorithm;
import learning.SarinVahidRL;
import library.ArrayOperations;
import library.MathOperations;
import library.ProfitCalculation;
import observer.ObserverLM2;
import observer.ObserverLM3;

public class DivisionManagerLM3 extends DivisionManager {
	protected String identifier;
	protected String stateKey;
	protected ObserverLM3 observer;
	protected Beliefs beliefs;
	protected FicticiousPlay la;
	
	/**
	 * Creates division manager agent for learning model LM1. 
	 * @param identifier - Identifier for the division manager agent
	 * @param reductProd - Productivity reduction
	 * @param optCapacity - Optimal capacity (threshold I)
	 * @param maxCapacity - Maximum capacity (threshold II)
	 * @param pMin - Minimum productivity value
	 * @param prodScaleRep - Scale of possible productivity values
	 * @param incrRep - Increment between productivity values
	 * @param verificationOutput - FileWriter to print output
	 * @param observer - Observer object
	 * @throws IOException 
	 */
	public DivisionManagerLM3(String identifier, double reductProd, int optCapacity, 
			int maxCapacity, double pMin, int prodScaleRep, double incrRep, 
			FileWriter verificationOutput, ObserverLM3 observer) throws IOException{
		
		super(reductProd, optCapacity,  maxCapacity,  pMin, prodScaleRep, incrRep, verificationOutput);
		this.identifier = identifier;							
		this.observer = observer;
		
		beliefs = new Beliefs(	ArrayOperations.generateArray(prodScaleRep, pMin, incrRep),
												MathOperations.roundTo4( (double) 1 / (double) prodScaleRep),
												identifier);
		
		la = new FicticiousPlay();
	}
	
	/* (non-Javadoc)
	 * @see agents.DivisionManager#reportProductivity(double)
	 */
	public double reportProductivity(double realP, double prediction) throws IOException{
		prodValue = realP;			
		//System.out.println("\n[reportProductivity, 64] realP: " + realP +
		//								"\nprediction: " + prediction);
		
		setStateKey(new String("" + prediction + realP));	// LM4 - prediction about repOpp
		//System.out.println("\n[reportProductivity, 64] StateKey: " + new String("" + prediction + realP));
		
		repProd =  la.respond(realP, prediction);
		
		//System.out.println("\n[reportProductivity,66] : " + repProd);
		return MathOperations.roundTo1(repProd);
	}
	
	/* (non-Javadoc)
	 * @see agents.DivisionManager#produce(int)
	 */
	public double produce(int resources){	
		resourcesAllocated = resources;
		//System.out.println("\n[produce,75] resourcesAllocated: " + resources);
		profit = ProfitCalculation.calcProfit(resources, prodValue, observer);
		//System.out.println("\n[produce,77] profit achieved: " + profit);
		return profit;
	}
	
	/* (non-Javadoc)
	 * @see agents.DivisionManager#receiveCompensation(double)
	 */
	public void receiveCompensation(double payment){
		compensation = payment;
		//System.out.println("\n[receiveCompensation, 86] receive compensation: " + compensation);
	}
	
	/**
	 * Updates the knowledge base of the division manager
	 * according to the observed report of the opponent 
	 *  
	 * @param observedReportOpponent
	 */
	public void updateKB(double observedReportOpponent){
		//System.out.println("\n[updateKB, 87_divmanager] observation: " + observedReportOpponent);
		la.updateKB(	observedReportOpponent, beliefs);
	}
	
	public String getIdentifier() {
		return identifier;
	}

	public String getStateKey() {
		return stateKey;
	}

	public void setStateKey(String stateKey) {
		this.stateKey = stateKey;
	}
	
	public FicticiousPlay getLa() {
		return la;
	}
	
	public Beliefs getBeliefs() {
		return beliefs;
	}

	public void setBeliefs(Beliefs beliefs) {
		this.beliefs = beliefs;
	}
}
