/** 
File: DivisonManagerZI.java
--------------------------------------------------------
Creates a division manager object for ZI learning model. 
* 
* Author:  Iris Lorscheid
* Date: May, 1st 2014 
*/

package agents;

import java.io.FileWriter;
import java.io.IOException;

import library.MathOperations;
import library.ProfitCalculation;

import observer.Observer;
import observer.ObserverZI;

public class DivisionManagerZI extends DivisionManager {
	private String identifier;
	ObserverZI observer;
	
	/**
	 * Creates division manager agent for ZI agents.
	 * 
	 * @param identifier
	 * @param reductProd
	 * @param optCapacity
	 * @param maxCapacity
	 * @param pMin
	 * @param prodScaleRep
	 * @param incrRep
	 * @param verificationOutput
	 * @param observer
	 */
	public DivisionManagerZI(String identifier, double reductProd, int optCapacity, 
			int maxCapacity, double pMin, int prodScaleRep, double incrRep,
			FileWriter verificationOutput, ObserverZI observer){
		
		super(reductProd, optCapacity,  maxCapacity,  pMin, prodScaleRep, incrRep, verificationOutput);
		this.identifier = identifier;							
		this.observer = observer;
	}
	

	/* (non-Javadoc)
	 * @see agents.DivisionManager#receiveCompensation(double)
	 */
	public void receiveCompensation(double payment){
		compensation = payment;
	}
	
	
	/* (non-Javadoc)
	 * @see agents.DivisionManager#reportProductivity(double)
	 */
	public double reportProductivity(double realP) throws IOException{
		prodValue = realP;																// here: no basis for the decision process
		int random = generator.nextInt(prodScaleRep);					// random value between 0 and n-1 (note: n-1!)
		repProd = (random * incrRep) + pMin;
		return MathOperations.roundTo1(repProd);
	}
	
	/* (non-Javadoc)
	 * @see agents.DivisionManager#produce(int)
	 */
	public double produce(int resources){	
		resourcesAllocated = resources;
		profit = ProfitCalculation.calcProfit(resources, prodValue, observer);
		return profit;
	}
	
	public String getIdentifier() {
		return identifier;
	}
	
}
