/** 
File: Headquarter.java
--------------------------------------------------------
Creates a headquarter object. 
* 
* Author:  Iris Lorscheid
* Date: May, 1st 2014 
*/


package agents;

import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import observer.Observer;
import library.ResourceAllocation;
import mechanism.GrovesMechanism;

public class Headquarter {

protected ArrayList<Integer> allocRes;	 
protected ArrayList<Double> repProd;
protected ArrayList<Double> compensations;
protected ArrayList<Double> profitDiv;
protected GrovesMechanism gm;
protected int resources, optCapacity, maxCapacity;
protected double totalP,reductProd, profit, discount;
private String identifier;
private Observer observer;
protected FileWriter verificationOutput;

/**
 * Creates headquarter agent
 * 
 * @param identifier
 * @param observer
 * @param resources
 * @param optCapacity
 * @param maxCapacity
 * @param reductProd
 * @param discount
 * @param verificationOutput
 */
public Headquarter(		String identifier, Observer observer,
											int resources, int optCapacity, 
											int maxCapacity, double reductProd, 
											double discount, FileWriter verificationOutput){
	
this.setIdentifier(identifier);;
this.setObserver(observer);
this.allocRes = new ArrayList<Integer>();
allocRes.add(0, 0);
allocRes.add(1, 0);

this.repProd = new ArrayList<Double>();
repProd.add(0,0.0);
repProd.add(1, 0.0);

this.compensations = new ArrayList<Double>();
compensations.add(0,0.0);
compensations.add(1, 0.0);

profitDiv = new ArrayList<Double>();
profitDiv.add(0,0.0);
profitDiv.add(1, 0.0);

this.resources = resources;
this.optCapacity = optCapacity;
this.maxCapacity = maxCapacity;

this.totalP = 0.0;
this.reductProd = reductProd;
this.setProfit(0.0);
this.discount = discount;

this.verificationOutput = verificationOutput;
}

/**
 * Prints setting for simulation run.
 * (debugging procedure)
 */
public void printSetting(){
	System.out.println(new String(
			"\nResources " + resources +
			"\noptCapacity " + optCapacity+
			"\nmaxCapacity " +maxCapacity+
			"\nreductProd " + reductProd
			));		
}

/**
 * Allocates resources to division manager agents, based on reports
 * -------------------------------------------------------------------------------------------------
 * @param report1 report2
 * @return  Number of resource units allocated for first report (report1)
 * @throws IOException
 */
public int allocResources(double report1, double report2) throws IOException{
	return ResourceAllocation.allocate(report1, report2, resources, observer);
}

/**
 * Calculates the company profit.
 * @param profit division1
 * @param profit division2
 * @return profit
 */
public double  calcCompanyProfit(double profit1, double profit2){
	profit = profit1 + profit2;
	profitDiv.set(0,profit1);
	profitDiv.set(1,profit2);
	return profit;
}

/**
 * Calculates the compensation for a division.
 * @param profit of division
 * @param expectProfitOpponent
 * @param discount for compensation scheme
 * @return compensation payment
 * @throws IOException
 */
public double payCompensation(	double profit, double expectProfitOpponent, double discount, boolean truthful) 
																throws IOException{
	return GrovesMechanism.calcCompensation(	profit, expectProfitOpponent, discount, truthful);
}

/**
 * Returns index with higher value.
 * @return {0,1}
 */
public int higherRep(){
	if(repProd.get(0) > repProd.get(1))
		return 0;
	else
		return 1;
}

/**
 * Returns index with lower value.
 * @return {0,1}
 */
public int lowerRep(){
	if(repProd.get(0) > repProd.get(1))
		return 1;
	else
		return 0;
}


public ArrayList<Integer> getAllocRes() {
	return allocRes;
}

public ArrayList<Double> getRepProd() {
	return repProd;
}

public int getResources() {
	return resources;
}
public void setResources(int resources) {
	this.resources = resources;
}
public int getOptCapacity() {
	return optCapacity;
}
public void setOptCapacity(int optCapacity) {
	this.optCapacity = optCapacity;
}
public int getMaxCapacity() {
	return maxCapacity;
}
public void setMaxCapacity(int maxCapacity) {
	this.maxCapacity = maxCapacity;
}
public double getTotalP() {
	return totalP;
}
public void setTotalP(double totalP) {
	this.totalP = totalP;
}
public double getReductProd() {
	return reductProd;
}
public void setReductProd(double reductProd) {
	this.reductProd = reductProd;
}

public double getProfit() {
	return profit;
}

public void setProfit(double profit) {
	this.profit = profit;
}

public ArrayList<Double> getCompensations() {
	return compensations;
}

public void setCompensations(ArrayList<Double> compensations) {
	this.compensations = compensations;
}

public String getIdentifier() {
	return identifier;
}

public void setIdentifier(String identifier) {
	this.identifier = identifier;
}

public Observer getObserver() {
	return observer;
}

public void setObserver(Observer observer) {
	this.observer = observer;
}

}
