/**
File: Window.java
------------------------------------------------------------------------------------------
Opens a window for defining the experimental design.
* 
* Author:  Iris Lorscheid
* Date: May, 1st 2014 
*/

package gui;
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;

import javax.swing.*;


import library.ParseOperations;
import main.LamdaScheduler;

public class Window  extends Frame {
	/**
	 * Universal ID to identify object in deserialization process
	 */
	private static final long serialVersionUID = 111L;
	
	private int w, h;
	private Checkbox ziCheckBox, lm1CheckBox, lm2CheckBox, lm2plusCheckBox, lm3CheckBox;
	private Button submitButton;
	private TextField f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, f12, f13, f14, f15, f16, f17, f18, f19;
	private Frame f = new Frame();
	private String 	factor2, factor3, factor4, factor5, factor6, factor7, factor8, factor9, 
								factor10, factor11, factor12, factor13, factor14, factor15, factor16,
								factor17, factor18, factor19;
	
	public Window(){
	}
	
	/**
	 * Creates window for input parameter values
	 * 
	 * @param w weight of window
	 * @param h height of window
	 */
	public void createWindow(int w, int h){
	this.w=w;
	this.h=h;
	
	this.setTitle("LAMDA - Design of Experiments");
	this.setSize(w,h);
	
	// centralize window on screen
	Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
	this.setLocation((d.width-w)/2, (d.height-h)/2);
	
	// for window action (closing)
	addWindowListener (new WindowClosingAdapter ());
	// for button action (klick)
	ActionListener startExp = new pressButton();
	
	// layout manager - x rows, y columns
	setLayout(new GridLayout(25,2));
	
	// check boxes for learning models
	add(new JLabel("<html><left> (1) Please choose one (or more) learning models:<p></html>")); 
	setCheckBoxes();
		
	f11 = new TextField("0,5", 6);
	add(new Label("Lamda - Reinforcement learning parameter"));
	add(f11);
			
	f12 = new TextField("64", 4);
	add(new Label("Number of times to explore each strategy systematically"));
	add(f12);
	
	add(new JSeparator());
		
	// design of experiments
	add(new JLabel("<html><left> (2) Please choose the scenario parameters:<p></html>")); 
	defineDesignPoints();
	
	add(new JSeparator());
	
	// opponent
	add(new JLabel("<html><left> (3) Please choose the opponents parameters:<p></html>")); 
	defineOpponentParameters();
	
	add(new JSeparator());
	
	// simulation parameters
	add(new JLabel("<html><left> (4) Please choose the experimental parameters:<p></html>")); 
	defineExpParameters();
	
	// submission button
	submitButton = new Button("Start simulation experiment!");
	add(submitButton);
	submitButton.addActionListener(startExp);
	
	this.setVisible(true);
	}
	
	/**
	 * Defines and adds check boxes to window
	 */
	public void setCheckBoxes(){
		ziCheckBox = new Checkbox("Zero Intelligence", false);
		lm1CheckBox = new Checkbox("LM 1 (Reinforcement Learning)", false);
		lm2CheckBox = new Checkbox("LM2 (Explicit experience collection)", true);
		lm2plusCheckBox = new Checkbox("LM2+ (Systematic exploration)", false);
		lm3CheckBox = new Checkbox("LM3 (Anticipate opponent behavior)",false);
		
		add(ziCheckBox);
		
		add(lm1CheckBox);
		add(lm2CheckBox);
		add(lm2plusCheckBox);
		add(lm3CheckBox);
		
		
	}
	
	/**
	 * Defines and adds input fields for factor level ranges
	 */
	public void defineDesignPoints(){
		f2 = new TextField("10 120 230",4);
		add(new Label("Resources - Integer values"));
		add(f2);
		
		f3 = new TextField("1,1 1,4 1,7",4);
		add(new Label("Min. productivity value - Double values"));
		add(f3);
		
		f4 = new TextField("3 8 13",4);
		add(new Label("Productivity scale - Integer values (length of strategy set, number of productivity values)"));
		add(f4);
		
		f17 = new TextField("0,1",4);
		add(new Label("Increment for real values - Double values"));
		add(f17);
		
		f18 = new TextField("<varied with prodScale>",4);
		add(new Label("Productivity scale for reported values - Integer values"));
		add(f18);
		
		f19 = new TextField("<varied with increment for real values>",4);
		add(new Label("Increment for reported values - Double values"));
		add(f19);
		
		f5 = new TextField("0,33" ,4);
		add(new Label("Optimal capacity - Double values out of [0,1] (share of resources) - Use comma as decimal"));
		add(f5);
		
		f6 = new TextField("0,83",4);
		add(new Label("Maximum capacity - Double values out of [0,1] (share of resources) - Use comma as decimal"));
		add(f6);
		
		f7 = new TextField("0,1 0,3 0,5",4);
		add(new Label("Productivity reduction - Double values out of [0,1] (share of reduced productivity - Use comma as decimal)"));
		add(f7);
		
		f8 = new TextField("0,1 0,3 0,5",4);
		add(new Label("Discount factor - Double values out of [0,1] - Use comma as decimal"));
		add(f8);
	}
	
	/**
	 * GUI for opponent parameter settings
	 */
	public void defineOpponentParameters(){
		f13 = new TextField("<varied with prodScale>",4);
		add(new Label("Opponent productivity scale - Integer values"));
		add(f13);
		
		f14 = new TextField("<varied with increment for real values>",4);
		add(new Label("Opponent increment for real values - Double values"));
		add(f14);
		
		f15 = new TextField("<varied with prodScale>",4);
		add(new Label("Opponent productivity scale for reported values - Integer values"));
		add(f15);
		
		f16 = new TextField("<varied with increment for real values>",4);
		add(new Label("Opponent increment for reported values - Double values"));
		add(f16);
	}
	
	/**
	 * Defines and adds input fields for experimental parameters (runs and ticks)
	 */
	public void defineExpParameters(){
		f9 = new TextField("3000",4);
		f10 = new TextField("10",4);
		
		add(new Label("Number of ticks per run - Integer values"));
		add(f9);
		add(new Label("Number of runs per setting - Integer values"));
		add(f10);
	}
	
	/**
	 * Reads and processes input from window
	 */
	public void readInput(){
		readLearningModels();
		readDesignOfExp();
		
		// write input in scheduler
		LamdaScheduler.setResources(ParseOperations.parseIntValues(factor2, "Resources (F2)"));
		LamdaScheduler.setPMin(ParseOperations.parseDoubleValues(factor3, "Minimum Productivity (F3)"));
		LamdaScheduler.setProdScale(ParseOperations.parseIntValues(factor4, "Productivity scale (F3)"));
		LamdaScheduler.setOptCapacity(ParseOperations.parseDoubleValues(factor5, "Optimal capacity (F4)"));
		LamdaScheduler.setMaxCapacity(ParseOperations.parseDoubleValues(factor6, "Maximum capacity (F5)"));
		LamdaScheduler.setReductProd(ParseOperations.parseDoubleValues(factor7, "ReductProductivity (F6)"));
		LamdaScheduler.setDiscount(ParseOperations.parseDoubleValues(factor8, "Discount factor"));
		LamdaScheduler.setNumberOfTicks(ParseOperations.parseIntValues(factor9, "Number of ticks per run"));
		LamdaScheduler.setNumberOfRuns(ParseOperations.parseIntValues(factor10, "Number of runs per setting"));
		LamdaScheduler.setLamda(ParseOperations.parseDoubleValues(factor11, "Lamda"));
		LamdaScheduler.setK(ParseOperations.parseIntValues(factor12, "k"));
		LamdaScheduler.setIncrRealP(ParseOperations.parseDoubleValues(factor17, "incrRealP"));
		
		if (factor13.contains("varied"))
			LamdaScheduler.setOpponentProdScale(LamdaScheduler.getProdScale());
		else
			LamdaScheduler.setOpponentProdScale(ParseOperations.parseIntValues(factor13, "oppProdScale"));
		 
		if (factor18.contains("varied"))
			LamdaScheduler.setProdScaleRep(LamdaScheduler.getProdScale());
		else
			LamdaScheduler.setProdScaleRep(ParseOperations.parseIntValues(factor18, "prodScaleRep"));
		 
		if (factor15.contains("varied"))
			LamdaScheduler.setOpponentProdScaleRep(LamdaScheduler.getProdScale());
		else
			LamdaScheduler.setOpponentProdScaleRep(ParseOperations.parseIntValues(factor15, "opponentProdScaleRep"));
		 
		if (factor14.contains("varied"))
			LamdaScheduler.setOpponentIncrRealP(LamdaScheduler.getIncrRealP());
		else
			LamdaScheduler.setOpponentIncrRealP(ParseOperations.parseDoubleValues(factor14, "opponentIncrRealP"));
		 
		if (factor19.contains("varied"))
			LamdaScheduler.setIncrRepP(LamdaScheduler.getIncrRealP());
		else
			LamdaScheduler.setIncrRepP(ParseOperations.parseDoubleValues(factor19, "incrRepP"));
		
		if (factor16.contains("varied"))
			LamdaScheduler.setOpponentIncrRepP(LamdaScheduler.getIncrRealP());
		else
			LamdaScheduler.setOpponentIncrRepP(ParseOperations.parseDoubleValues(factor16, "opponentIncrRepP"));
		
	}
	
	/**
	 * Reads factor level ranges
	 */
	public void readDesignOfExp(){
		factor2 = f2.getText();
		factor3 = f3.getText();
		factor4 = f4.getText();
		factor5 = f5.getText();
		factor6 = f6.getText();
		factor7 = f7.getText();
		factor8 = f8.getText();
		factor9 = f9.getText();
		factor10 = f10.getText();
		factor11 = f11.getText();
		factor12 = f12.getText();
		factor13 = f13.getText();
		factor14 = f14.getText();
		factor15 = f15.getText();
		factor16 = f16.getText();
		factor17 = f17.getText();
		factor18 = f18.getText();
		factor19 = f19.getText();
	}
	
	/**
	 * Reads chosen learning models
	 */
	public void readLearningModels(){
		if (ziCheckBox.getState() )
					LamdaScheduler.setZi(true);
		if (lm1CheckBox.getState() )
					LamdaScheduler.setLm1(true);
		if (lm2CheckBox.getState() )
					LamdaScheduler.setLm2(true);
		if (lm2plusCheckBox.getState() )
					LamdaScheduler.setLm2plus(true);
		if (lm3CheckBox.getState() )
					LamdaScheduler.setLm3(true);
		if(!ziCheckBox.getState() && !lm1CheckBox.getState() && !lm2CheckBox.getState() 
						&& !lm2plusCheckBox.getState() && !lm3CheckBox.getState())
		{
					JOptionPane.showMessageDialog(f,
						    "No learning model chosen, please restart!",
						    "LAMDA - error message",
						    JOptionPane.ERROR_MESSAGE);
					System.exit(0);
				}
	}
	
	// Internal support class for closing the window.
	public class WindowClosingAdapter extends WindowAdapter {
		public void windowClosing( WindowEvent e ){
			System.exit(0);
		}
	}
	
	// Internal support class for react on the button.
	public class pressButton implements ActionListener{
		public void actionPerformed(ActionEvent e){
		readInput();
		try {
			LamdaScheduler.scheduleLAMDA();
		} catch (IOException e1) {
			JOptionPane.showMessageDialog(f,
				    "IO Exception. Please try again.",
				    "LAMDA - error message",
				    JOptionPane.ERROR_MESSAGE);
			e1.printStackTrace();
		}
		Window.this.dispose();
		}
	}
	
}
