/**
File: Knowledgebase.java
------------------------------------------------------------------------------------------
* Implements the agents knowledge base.
* 
* Author:  Iris Lorscheid
* Date: May, 1st 2014 
*/

package knowledgebase;

import java.util.Hashtable;

public class Knowledgebase {

	protected String id;
	
	/**	Hashtable of states */
	protected Hashtable<String,State> states;
	
	/** Possible actions for strategy set 
	 *	Assumption: Fix action set for whole simulation run */
	protected double[] actions;
	
	/** Initial strength for strategy set */
	protected double initial_strength;
	
	/** Counter for the number of states in given knowledge base*/
	protected int state_counter;

	public Knowledgebase(){
		
	}
	
	/**
	 * Constructor
	 * 
	 * @param actions			-- actions for strategy set
	 * @param initial_strength	-- initial strength for strategy set
	 */
	public Knowledgebase(	double[] actions, 
							double initial_strength,
							String agent_identifier)
	{
		this.actions = actions;
		this.initial_strength = initial_strength;
		this.id = new String("KB_" + agent_identifier);
		states = new Hashtable<String,State>();
		state_counter = 0;
	}
	
	/**
	 * Returns decision from given knowledge base
	 * @param state_key 	-- current state
	 * @return action		-- decision
	 */
	public double ask(String state_key)
	{
		if (!checkState(state_key))
		{
			createState(state_key);
		} 
		return states.get(state_key).selectActionStochastically();
	}
	
	/**
	 * Checks whether a given state exists in Hashtable
	 * @param state_key
	 * @return
	 */
	public boolean checkState(String state_key)
	{
		if(states.containsKey(state_key))
			return true;
		else
			return false;
	}
	
	/**
	 * Generates a new state with current state key in Hashtable
	 * @param state_key
	 */
	public void createState(String state_key)
	{
		state_counter++;
		states.put(state_key, new State(
										new String(id + state_key),
										actions,
										initial_strength) );
	}

	
	/**
	 * Returns truthful decision from given knowledge base
	 * @param state_key 	-- current state
	 * @return action		-- decision
	 */
	public double ask_truthful(String state_key)
	{
		if (!checkState(state_key))
		{
			createState(state_key);
		} 
		return states.get(state_key).selectActionTruthfully();
	}
	
	// GETTERS AND SETTERS

	public Hashtable<String, State> getStates() {
		return states;
	}
	public void setStates(Hashtable<String, State> states) {
		this.states = states;
	}
	public double[] getActions() {
		return actions;
	}
	public void setActions(double[] actions) {
		this.actions = actions;
	}
	public double getInitial_strength() {
		return initial_strength;
	}
	public void setInitial_strength(double initialStrength) {
		initial_strength = initialStrength;
	}
	public int getState_counter() {
		return state_counter;
	}
	public void setState_counter(int stateCounter) {
		state_counter = stateCounter;
	}
	
}
