/**
* File: Strategy.java
* --------------------------------------------------------
* Creates a strategy object for the knowledge base 
* (action set and strength values for perceived states). 
* 
* Author:  Iris Lorscheid
* Date: May, 1st 2014 
*
*/
package knowledgebase;

import library.*;

public class Strategy {
	
	/** unique identifier [state;action] */
	private String id;
	/** action of strategy */
	private double action;
	/** strength of strategy*/
	private double strength;
	
	/**
	 * Constructor
	 * @param id 			- unique identifier for strategy
	 * @param action	- action for strategy
	 * @param init_strength - initial strength for strategy
	 */
	public Strategy(String id, 
					double action, 
					double init_strength){
		this.setId(id); 				
		this.action = action;
		this.strength = init_strength;
		}
	
	/**
	 * Calculates selection probability based on given strength 
	 * and sum of strength over the strategy set
	 * @param strength_sum	- sum of strength over strategy set
	 * @return probability	- rounded to 4 decimal places
	 */
	public double calcSelectionProbability(double strength_sum){
		double prob = 0.0;
		if(strength< 0){
			prob =  0.0;
		} else{
			prob = MathOperations.roundTo4(strength/strength_sum);
		}
		return prob;
	}
	
	public double getAction() {
		return action;
	}
	public void setAction(double action) {
		this.action = action;
	}
	public double getStrength() {
		return strength;
	}
	public void setStrength(double strength) {
		this.strength = strength;
	}
	public void setId(String id) {
		this.id = id;
	}
	public String getId() {
		return id;
	}
}
