/**
* 
* File: SarinVahidRL.java
* ---------------------------------------------
* Implementation of the SV learning algorithm.
* 
* Author:  Iris Lorscheid
* Date: May, 1st 2014 
*/

package learning;

import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;


import knowledgebase.*;
import library.MathOperations;

public class SarinVahidRL extends LearningAlgorithm{
	int count;
	private FileWriter writeStrengthValues;

	public SarinVahidRL(String identifier) throws IOException {
		super();
		count = 0;
		setWriteStrengthValues(new FileWriter("strengthValues"+ identifier + ".csv"));
		writeStrengthValues.append("state;1.4;1.5;1.6;1.7;1.8;1.9;2.0;2.1");
		// TODO Auto-generated constructor stub
	}
	
	private double lambda;
	
	/* (non-Javadoc)
	 * @see learning.LearningAlgorithm#updateKB(knowledgebase.State, java.lang.String, double)
	 */
	public void updateKB(	State s,
							String action_key, 
							double feedback
							) throws IOException
	{
		count++;
		
		double utility = 0.0; 		// utility at t
		double utility_next = 0.0;	// utility at t+1
		
		// (1) Calculating new utility for chosen strategy
		utility = s.getStr().get(action_key).getStrength();
		//System.out.println(	"\n [DEBUGGING STRENGTH-UPDATE] " +	"\nOriginal strength chosen action "	+  action_key + " : "+ utility);	
		
		utility_next = 	
							MathOperations.roundTo4(
							(1-lambda) * utility 
							+
							lambda * feedback
							);
		
		if (utility_next < 0.01){
			utility_next = 0.01;
		}
		
		/*System.out.println("\n Utility old (chosen action): " + utility
				+ "\n Feedback: " + feedback
				+ "\n Lamda: " + lambda +
				"\n New strength chosen action "	+  action_key + " : "+ utility_next);*/

		// (2) New utility for chosen action
		s.getStr().get(action_key).setStrength(utility_next);
		
		// (3) New utility for not chosen actions
		updateStrategies(s);
	}
	
	private void updateStrategies(State s) throws IOException {
		double strength_sum = 0.0;
		double updatePosShare= 0.0;
		double sumNegative = 0.0;
		double updateNegShare = 0.0;
		int countStrategies = 0;
		
		 if(count < 2999)
		  writeStrengthValues.append("\n" + s + ";");
		
		 
		// check: sum of strength values in [0,1]?
		Enumeration<Strategy> e0 = s.getStr().elements();
		Strategy tmp0 = null;
		while( e0.hasMoreElements())
		{
		 tmp0 = e0.nextElement();
		 countStrategies++;
		 strength_sum += tmp0.getStrength();
		}
		 	
		// sum > 1.0
		if (strength_sum > 1.0){
			updatePosShare = MathOperations.roundTo4( (strength_sum - 1.0) / (double) countStrategies ); 
			//System.out.println("\n	Update posShare: 	" + updatePosShare );	
			
			Enumeration<Strategy> e1 = s.getStr().elements();
			Strategy tmp1 = null;
			while( e1.hasMoreElements())
			{
			 tmp1 = e1.nextElement();
			 tmp1.setStrength(tmp1.getStrength() - updatePosShare);
			 
			 if(tmp1.getStrength() < 0.0){
				 sumNegative -= 0.0 - tmp1.getStrength();
				 tmp1.setStrength(0.0);
			 }
			}
		}
		
		if(sumNegative > 0.0){
			updateNegShare = MathOperations.roundTo4( sumNegative / (double) countStrategies ); 
			//System.out.println("\n	Update negShare: 	" + updateNegShare );	
			
			Enumeration<Strategy> e2 = s.getStr().elements();
			Strategy tmp2 = null;
			while( e2.hasMoreElements())
			{
			 tmp2 = e2.nextElement();
			 if (tmp2.getStrength() > updateNegShare){
				 tmp2.setStrength(tmp2.getStrength() - updateNegShare);
				 } else{
					 tmp2.setStrength(0.0);
				 }
			}
		}
		
			 
		if(count == 2999 ){
			 writeStrengthValues.close();
		}
	}	

	public void setLambda(double lambda) {
		this.lambda = lambda;
	}
	public double getLambda() {
		return lambda;
	}

	public FileWriter getWriteStrengthValues() {
		return writeStrengthValues;
	}

	public void setWriteStrengthValues(FileWriter writeStrengthValues) {
		this.writeStrengthValues = writeStrengthValues;
	}	
}
