/**
* File: ResourceAllocation.java
* --------------------------------------------------------
* Calculates the optimal resource allocation given the productivity values of units 
* and resources available. 
* 
* Author:  Iris Lorscheid
* Date: May, 1st 2014 
*
*/


package library;

import java.util.Hashtable;
import java.util.Random;
import observer.Observer;

public class ResourceAllocation {
	static Random generator;

	/**
	 * Allocates resource units to division with pValue1
	 * ----------------------------------------------------------------------
	 * @param pValue1 -- own
	 * @param pValue2 -- opponent
	 * @param resources -- sum of res units
	 * @param observer -- observer (parameter values)
	 * @return number of resource units allocated
	 */
	public static int allocate(double pValue1, double pValue2, int resources, Observer observer){
		Hashtable<String, Double> values = new Hashtable<String, Double>();
		values.put("lowValue", lowerValue(pValue1, pValue2));
		values.put("highValue",higherValue(pValue1, pValue2));
		
		double difference =Math.abs(pValue1 - pValue2);
		if( difference < 0.001){
			return uniformAllocation(resources);
		} 	
		
		else {
			if(		values.get("lowValue") > ( values.get("highValue") - observer.getReductProd() ) 	
					|| 					
					Math.abs( difference- observer.getReductProd() )  < 0.001 ){
					
					if	( pValue1 == values.get("lowValue"))
						return allocation1Low(values, resources, observer);
					else
						return allocation1High(values, resources, observer);
			} 
			else { 
				if	( pValue1 == values.get("lowValue"))
					return allocation2Low(values, resources, observer);
				else
					return allocation2High(values, resources, observer);
			}
		} 
	}
	
	public static int allocation1Low(Hashtable<String, Double> values, int resources, Observer observer){
		int rest = 0; 				// resources above maxCapacity of division with higher productivity
		int restShare = 0;		// resources above both maxCapacity (to be shared)
		
		if ( 	resources <= observer.getOptCapacity() ) {
			return 0;
		}
		
		else if(	 	resources - observer.getOptCapacity()	<= observer.getOptCapacity()) {
			return resources - observer.getOptCapacity() ;
		} 
		
		else if (	resources - observer.getOptCapacity() <= observer.getMaxCapacity()) {	//&& > optCapacity (by else)
			return observer.getOptCapacity();	
		} 
		
		else if ( 	resources - observer.getOptCapacity() > observer.getMaxCapacity()
						&&
						resources <=	2* observer.getMaxCapacity()) {	
			rest = resources - observer.getMaxCapacity() - observer.getOptCapacity();
			return observer.getOptCapacity() + rest;				
		} 
		
		else if( 	resources > 2 * observer.getMaxCapacity() ) { 
			restShare = resources - 2*observer.getMaxCapacity();
			return observer.getMaxCapacity()+ restShare;		
		}
		
		return -1;
	}
	
	public static int allocation1High(Hashtable<String, Double> values, int resources, Observer observer){
		int restShare = 0;		// resources above both maxCapacity (to be shared)
		
		if ( 	resources <= observer.getOptCapacity() ) {
			return  observer.getOptCapacity();
		}
		
		else if(	 	resources - observer.getOptCapacity()	<= observer.getOptCapacity()) {
			return observer.getOptCapacity() ;
		} 
		
		else if (	resources - observer.getOptCapacity() <= observer.getMaxCapacity()) {	//&& > optCapacity (by else)
			return resources - observer.getOptCapacity();	
		} 
		
		else if ( 	resources - observer.getOptCapacity() > observer.getMaxCapacity()
						&&
						resources <=	2* observer.getMaxCapacity()) {	
			return observer.getMaxCapacity();				
		} 
		
		else if( 	resources > 2 * observer.getMaxCapacity() ) { 
			restShare = resources - 2*observer.getMaxCapacity();
			return observer.getMaxCapacity()+ restShare;		
		}
		
		return -1;
	}
	
	public static int allocation2Low(Hashtable<String, Double> values, int resources, Observer observer){
		int restShare = 0;
		
		if(		resources <= observer.getMaxCapacity()){
			return 0;	
		}
		
		else if(		resources > observer.getMaxCapacity()
						&&
						resources <= 2 * observer.getMaxCapacity() ) {
			return resources - observer.getMaxCapacity();	
		}  
		
		else if(		resources > 2 * observer.getMaxCapacity() )  {
			 restShare = resources - 2*observer.getMaxCapacity();
			 return observer.getMaxCapacity() + restShare/2;	
		}
		
		return -1;
	}
	
	public static int allocation2High(Hashtable<String, Double> values, int resources, Observer observer){
		int restShare = 0;
		
		if(	resources <= observer.getMaxCapacity()){
			return resources;	
		}
		
		else if(		resources > observer.getMaxCapacity()
						&&
						resources <= 2 * observer.getMaxCapacity() ) {
			return observer.getMaxCapacity();	
		}  
		
		else if(		resources > 2 * observer.getMaxCapacity() )  {
			 restShare = resources - 2*observer.getMaxCapacity();
			 return observer.getMaxCapacity() + restShare/2;		
		}
		
		return -1;
	}
	
	public static int uniformAllocation (int resources){
		if( resources % 2 != 0){	
			generator = new Random();
			int rand = generator.nextInt(2);
			if(rand == 1)
				return resources / 2 +1;
		}
		return resources / 2;
	}
	
	
	/**
	 * Returns higher value.
	 * --------------------------------
	 */
	public static double higherValue(double v1, double v2){
		if(v1 >= v2)
			return v1;
		else
			return v2;
	}

	/**
	 * Returns lower value.
	 * ------------------------------
	 */
	public static double lowerValue(double v1, double v2){
		if(v1 < v2)
			return v1;
		else
			return v2;
	}

}
