/**
* File: LamdaScheduler.java
* --------------------------------------------------------
* Organizes and schedules the simulation runs according to the design of experiment. 
* 
* Author:  Iris Lorscheid
* Date: May, 1st 2014 
*
*/

package main;

import observer.*;

import java.awt.Component;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import javax.swing.JOptionPane;

public class LamdaScheduler {

	private static boolean zi, lm1, lm2, lm2plus, lm3;
	private static ArrayList<Integer> 	resources, prodScale, opponentProdScale, 
																prodScaleRep, opponentProdScaleRep,
																numberOfRuns, numberOfTicks, k;
	private static ArrayList<Double> 	pMin, optCapacity, maxCapacity, 
																incrRealP, opponentIncrRealP,
																incrRepP, opponentIncrRepP,
																reductProd, discount, lamda;
	private static int designPoint;

	private static Component f;
	private static ArrayList<ObserverZI> obsZI;
	private static ArrayList<ObserverLM1> obsLM1;
	private static ArrayList<ObserverLM2> obsLM2;
	private static ArrayList<ObserverLM2plus> obsLM2plus;	
	private static ArrayList<ObserverLM3> obsLM3;	
	private static FileWriter outputCSV;
	
	/**
	 *  Schedules simulation experiments
	 * @throws IOException 
	 */
	public static void scheduleLAMDA() throws IOException {
		printDesignPoints();
		initializeObjects();
		initializeOutputFile();
		writeHeadline();
		
		designPoint = 0;
		
		for(int ind1= 0; ind1 < resources.size(); ind1++){ 												//[for1]
				for(int ind2= 0; ind2 < pMin.size(); ind2++){												//[for2]
					for(int ind3= 0; ind3 <prodScale.size(); ind3++){									//[for3]
						for(int ind4= 0; ind4 < optCapacity.size(); ind4++){							//[for4]
							for(int ind5= 0; ind5 < reductProd.size() ; ind5++){						//[for5]
								for(int ind6= 0; ind6 < discount.size(); ind6++){							//[for6]
									for(int ind7 = 0; ind7 < numberOfTicks.size(); ind7++){		//[for7]
										for(int ind8 = 0; ind8 < numberOfRuns.size();ind8++){		//[for8]
											for(int ind9 = 0; ind9 < lamda.size();ind9++){				//[for9]
												for(int ind10 = 0; ind10 < k.size();ind10++){				//[for10]
													for(int ind11 = 0; ind11 < opponentProdScale.size();ind11++){							//[for11]
														for(int ind12 = 0; ind12 < prodScaleRep.size();ind12++){								//[for12]
															for(int ind13 = 0; ind13 < opponentProdScaleRep.size();ind13++){			//for13]
																for(int ind14 = 0; ind14 < incrRealP.size();ind14++){									//[for14]
																	for(int ind15 = 0; ind15 < opponentIncrRealP.size();ind15++){			//[for15]
																		for(int ind16 = 0; ind16 < incrRepP.size();ind16++){							//[for16]
																			for(int ind17 = 0; ind17 < opponentIncrRepP.size();ind17++){	//[for17]
																				
										designPoint++;
										
										if(zi)
										{
											System.out.println("\ndesignPoint - ZI: " + designPoint);
											obsZI.add( new ObserverZI(new String(ind1+"-" + ind2 + "-"+ ind3 + "-"+ ind4 + "-"+ ind5 + "-"+ ind6),	
																				resources.get(ind1),
																				pMin.get(ind2),
																				prodScale.get(ind3),
																				opponentProdScale.get(ind11),
																				incrRealP.get(ind14), 
																				opponentIncrRealP.get(ind15), 
																				prodScale.get(ind3), 
																				prodScale.get(ind3),
																				incrRepP.get(ind16), 
																				opponentIncrRepP.get(ind17), 	
																				optCapacity.get(ind4),	// opt & max capacity build a value pair
																				maxCapacity.get(ind4), // only variation over common indices
																				reductProd.get(ind5),
																				discount.get(ind6),
																				numberOfTicks.get(ind7),
																				numberOfRuns.get(ind8),
																				outputCSV,
																				designPoint
																				));
											} // ZI
											
											if(lm1){
												System.out.println("\ndesignPoint - LM1: " + designPoint 
														+ "\nlamda: " + lamda.get(ind9));
												obsLM1.add( new ObserverLM1(new String(ind1+"-" + ind2 + "-"+ ind3 + "-"+ ind4 + "-"+ ind5 + "-"+ ind6),	
																			resources.get(ind1),
																			pMin.get(ind2),
																			prodScale.get(ind3),
																			prodScale.get(ind3),					//opponentProdScale.get(ind11),
																			incrRealP.get(ind14), 
																			opponentIncrRealP.get(ind15), 
																			prodScale.get(ind3), //prodScaleRep.get(ind12), 
																			prodScale.get(ind3), //opponentProdScaleRep.get(ind13),
																			incrRepP.get(ind16), 
																			opponentIncrRepP.get(ind17), 	
																			optCapacity.get(ind4),	// opt & max capacity build a value pair
																			maxCapacity.get(ind4), // only variation over common indices
																			reductProd.get(ind5),
																			discount.get(ind6),
																			numberOfTicks.get(ind7),
																			numberOfRuns.get(ind8),
																			outputCSV,
																			designPoint,
																			lamda.get(ind9)
														));
											} //lm1
											
											if(lm2){
												System.out.println("\ndesignPoint - LM2: " + designPoint + "\nlamda: " + lamda.get(ind9));
												obsLM2.add( new ObserverLM2(new String(ind1+"-" + ind2 + "-"+ ind3 + "-"+ ind4 + "-"+ ind5 + "-"+ ind6),	
														resources.get(ind1),
														pMin.get(ind2),
														prodScale.get(ind3),
														prodScale.get(ind3),					//opponentProdScale.get(ind11),
														incrRealP.get(ind14), 
														opponentIncrRealP.get(ind15), 
														prodScale.get(ind3), //prodScaleRep.get(ind12), 
														prodScale.get(ind3), //opponentProdScaleRep.get(ind13),
														incrRepP.get(ind16), 
														opponentIncrRepP.get(ind17), 	
														optCapacity.get(ind4),	// opt & max capacity build a value pair
														maxCapacity.get(ind4), // only variation over common indices
														reductProd.get(ind5),
														discount.get(ind6),
														numberOfTicks.get(ind7),
														numberOfRuns.get(ind8),
														outputCSV,
														designPoint,
														lamda.get(ind9)
														));
											} //lm2
											
											if(lm2plus){
												System.out.println("\ndesignPoint - LM3: " + designPoint + "\nlamda: " + lamda.get(ind9)
														+ " k: " + k);
												obsLM2plus.add( new ObserverLM2plus(new String(ind1+"-" + ind2 + "-"+ ind3 + "-"+ ind4 + "-"+ ind5 + "-"+ ind6),	
														resources.get(ind1),
														pMin.get(ind2),
														prodScale.get(ind3),
														prodScale.get(ind3),					//opponentProdScale.get(ind11),
														incrRealP.get(ind14), 
														opponentIncrRealP.get(ind15), 
														prodScale.get(ind3), //prodScaleRep.get(ind12), 
														prodScale.get(ind3), //opponentProdScaleRep.get(ind13),
														incrRepP.get(ind16), 
														opponentIncrRepP.get(ind17), 	
														optCapacity.get(ind4),	// opt & max capacity build a value pair
														maxCapacity.get(ind4), // only variation over common indices
														reductProd.get(ind5),
														discount.get(ind6),
														numberOfTicks.get(ind7),
														numberOfRuns.get(ind8),
														outputCSV,
														designPoint,
														lamda.get(ind9),
														k.get(ind10)
														));
											} // lm2plus				
											
											if(lm3){
												System.out.println("\ndesignPoint - LM3: " + designPoint + "\nlamda: " + lamda.get(ind9));
												obsLM3.add( new ObserverLM3(new String(ind1+"-" + ind2 + "-"+ ind3 + "-"+ ind4 + "-"+ ind5 + "-"+ ind6),	
														resources.get(ind1),
														pMin.get(ind2),
														prodScale.get(ind3),
														prodScale.get(ind3),				// opponentProdScale.get(ind11), //
														incrRealP.get(ind14), 
														opponentIncrRealP.get(ind15), 
														prodScale.get(ind3), //prodScaleRep.get(ind12), 
														prodScale.get(ind3), // opponentProdScaleRep.get(ind13), //
														incrRepP.get(ind16), 
														opponentIncrRepP.get(ind17), 	
														optCapacity.get(ind4),	// opt & max capacity build a value pair
														maxCapacity.get(ind4), // only variation over common indices
														reductProd.get(ind5),
														discount.get(ind6),
														numberOfTicks.get(ind7),
														numberOfRuns.get(ind8),
														outputCSV,
														designPoint,
														lamda.get(ind9)
														));
											} //lm3
											
		} 								//[for1]
		} 								//[for2]
		} 								//[for3]
		} 								//[for4]
		}	 							//[for5]
		} 								//[for6]
		}								//[for7]
		}								//[for8]
		}								//[for9]
		}								// [for10]
		} 								// [for11]
		} 								// [for12]
		} 								// [for13]
		} 								// [for14]
		} 								// [for15]
		} 								// [for16]
		} 								// [for17]
		
		outputCSV.close();
		
	} 								// end of method
	
	/**
	 * Creates an ArrayList for the organization of agents.
	 */
	public static void initializeObjects() {
		obsZI = new ArrayList<ObserverZI>();
		obsLM1 = new ArrayList<ObserverLM1>();
		obsLM2 = new ArrayList<ObserverLM2>();
		obsLM2plus = new ArrayList<ObserverLM2plus>();
		obsLM3 = new ArrayList<ObserverLM3>();
	}
	
	/**
	 * This method initializes the csv output file (name of file: "Verification + Date + Hours + Minutes.csv").
	 * @throws IOException
	 */
	public static void initializeOutputFile() throws IOException{
		Date d = new Date();
		StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append("Output");
		stringBuilder.append(d.getDate());
		stringBuilder.append(d.getHours());
		stringBuilder.append(d.getMinutes());
		stringBuilder.append(".csv");
		outputCSV = new FileWriter(
													new String(stringBuilder.toString())
												);
	}
	
	/**
	 * Writes the headline in the output file
	 * @throws IOException
	 */
	public static void writeHeadline() throws IOException{
	outputCSV.append(	
			"tick" +
			";run" +
			";realProd1" +  
			";repProd1" +  
			";delta1"+	
			";relDelta1" + 
			";resAlloc1"+
			";profitDiv1"+
			";compensation1" +
			";relCompensation1" +
			
			";realProd2 "+
			";repProd2 "+ 
			";delta2"+ 
			";relDelta2"+
			";resAlloc2"+
			";profitDiv2"+
			";compensation2"+
			";relCompensation2" + 
			
			";profit" + 
			";trueProfit" + 
			";loss"+ 
			";hits"+ 			
			
			";resources "+ 
			";minValue" +
			";prodScale"+
			";optCap" + 
			";maxCap" +
			";reductProd" + 
			";discount" +
			//	";hypoProfit"+
			";lamda"+
			";k"+
			";lm" +
			";runs"+
			";ticks" +
			";belief_divM1" +
			";belief_divM2" +
			";maxCompens_divM1" +
			";maxCompens_divM2"
			
			);
	}
	
	/***
	 *  Prints information about the chosen design points
	 */
	public static void printDesignPoints(){
		
		int numberOfSettings = 	countLM() * resources.size() * pMin.size() *  prodScale.size() * optCapacity.size() 
													* reductProd.size() * discount.size() * lamda.size() * k.size();
		// + runs and settings
		
		// optCapacity and maxCapacity are dependent 
		// only 1 setting per opt-maxCapacity pair 
		
		JOptionPane.showMessageDialog(f,
			    new String("Thank you. You chose " + numberOfSettings + " design points-"
			    		+ " The simulation process starts."),
			    "LAMDA - information message",
			     JOptionPane.INFORMATION_MESSAGE);
	}
	
	/**
	 * @return number of chosen learning models
	 */
	public static int countLM(){
		int counter =0;
		if (zi)
			counter++;
		if (lm1)
			counter++;
		if(lm2)
			counter++;
		if(lm2plus)
			counter++;
		if(lm3)
			counter++;
		return counter;
	}

	public static ArrayList<Double> getDiscount() {
		return discount;
	}

	public static void setDiscount(ArrayList<Double> discount) {
		LamdaScheduler.discount = discount;
	}
	
	public static boolean isZi() {
		return zi;
	}

	public static void setZi(boolean zi) {
		LamdaScheduler.zi = zi;
	}

	public static boolean isLm1() {
		return lm1;
	}

	public static void setLm1(boolean lm1) {
		LamdaScheduler.lm1 = lm1;
	}

	public static boolean isLm2() {
		return lm2;
	}

	public static void setLm2(boolean lm2) {
		LamdaScheduler.lm2 = lm2;
	}

	public static boolean isLm2plus() {
		return lm2plus;
	}

	public static void setLm2plus(boolean lm2plus) {
		LamdaScheduler.lm2plus = lm2plus;
	}

	public static boolean isLm3() {
		return lm3;
	}

	public static void setLm3(boolean lm3) {
		LamdaScheduler.lm3 = lm3;
	}
	
	public static ArrayList<Integer> getNumberOfRuns() {
		return numberOfRuns;
	}

	public static void setNumberOfRuns(ArrayList<Integer> numberOfRuns) {
		LamdaScheduler.numberOfRuns = numberOfRuns;
	}

	public static ArrayList<Integer> getNumberOfTicks() {
		return numberOfTicks;
	}

	public static void setNumberOfTicks(ArrayList<Integer> numberOfTicks) {
		LamdaScheduler.numberOfTicks = numberOfTicks;
	}
	public static ArrayList<Integer> getResources() {
		return resources;
	}
	
	public static void setResources(ArrayList<Integer> resources) {
		LamdaScheduler.resources = resources;
	}
	
	public static ArrayList<Double> pMin() {
		return pMin;
	}

	public static void setPMin(ArrayList<Double> pMin) {
		LamdaScheduler.pMin = pMin;
	}
	
	public static ArrayList<Integer> getProdScale() {
		return prodScale;
	}

	public static void setProdScale(ArrayList<Integer> prodScale) {
		LamdaScheduler.prodScale = prodScale;
	}

	public static ArrayList<Double> getOptCapacity() {
		return optCapacity;
	}

	public static void setOptCapacity(ArrayList<Double> optCapacity) {
		LamdaScheduler.optCapacity = optCapacity;
	}

	public static ArrayList<Double> getMaxCapacity() {
		return maxCapacity;
	}

	public static void setMaxCapacity(ArrayList<Double> maxCapacity) {
		LamdaScheduler.maxCapacity = maxCapacity;
	}

	public static ArrayList<Double> getReductProd() {
		return reductProd;
	}

	public static void setReductProd(ArrayList<Double> reductProd) {
		LamdaScheduler.reductProd = reductProd;
	}

	public static ArrayList<Double> getLamda() {
		return lamda;
	}

	public static void setLamda(ArrayList<Double> lamda) {
		LamdaScheduler.lamda = lamda;
	}

	public static ArrayList<Integer> getK() {
		return k;
	}

	public static void setK(ArrayList<Integer> k) {
		LamdaScheduler.k = k;
	}

	public static ArrayList<Integer> getOpponentProdScale() {
		return opponentProdScale;
	}

	public static void setOpponentProdScale(ArrayList<Integer> opponentProdScale) {
		LamdaScheduler.opponentProdScale = opponentProdScale;
	}
	public static ArrayList<Integer> getProdScaleRep() {
		return prodScaleRep;
	}

	public static void setProdScaleRep(ArrayList<Integer> prodScaleRep) {
		LamdaScheduler.prodScaleRep = prodScaleRep;
	}

	public static ArrayList<Integer> getOpponentProdScaleRep() {
		return opponentProdScaleRep;
	}

	public static void setOpponentProdScaleRep(
			ArrayList<Integer> opponentProdScaleRep) {
		LamdaScheduler.opponentProdScaleRep = opponentProdScaleRep;
	}

	public static ArrayList<Double> getIncrRealP() {
		return incrRealP;
	}

	public static void setIncrRealP(ArrayList<Double> incrRealP) {
		LamdaScheduler.incrRealP = incrRealP;
	}

	public static ArrayList<Double> getOpponentIncrRealP() {
		return opponentIncrRealP;
	}

	public static void setOpponentIncrRealP(ArrayList<Double> opponentIncrRealP) {
		LamdaScheduler.opponentIncrRealP = opponentIncrRealP;
	}

	public static ArrayList<Double> getIncrRepP() {
		return incrRepP;
	}

	public static void setIncrRepP(ArrayList<Double> incrRepP) {
		LamdaScheduler.incrRepP = incrRepP;
	}

	public static ArrayList<Double> getOpponentIncrRepP() {
		return opponentIncrRepP;
	}

	public static void setOpponentIncrRepP(ArrayList<Double> opponentIncrRepP) {
		LamdaScheduler.opponentIncrRepP = opponentIncrRepP;
	}

}
