/**
* File: Observer.java
* --------------------------------------------------------
* Superclass for the Observer classes (scheduling and reporting).
* 
* Author:  Iris Lorscheid
* Date: May, 1st 2014 
*
*/

package observer;

import java.awt.Component;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Random;
import javax.swing.JOptionPane;

import library.MathOperations;
import library.ProfitCalculation;
import library.ResourceAllocation;

import agents.Headquarter;

public class Observer {
	// input variables
	protected int resources, prodScale, opponentProdScale, 
							prodScaleRep, opponentProdScaleRep,
							numberOfTicks, numberOfRuns, designPoint;
	protected double	 reductProd, discount, pMin, pMax, optCapacity, maxCapacity,
										incrRealP, opponentIncrRealP,
										incrRepP, opponentIncrRepP;
	public double getDiscount() {
		return discount;
	}

	protected int absOptCapacity, absMaxCapacity;
	// output measures
	protected double delta1, relDelta1, relCompensation1;
	protected double delta2, relDelta2, relCompensation2;
	protected double profit, loss;
	protected int hits;
	//protected int counter = 0;
	
	// support
	protected Random generator;
	protected Component f;
	protected FileWriter verificationOutput, paymentMatrix;
	protected Headquarter hqAgent;
	protected String identifier;
	
	public Observer( 	int resources, double pMin, 
										int prodScale, int opponentProdScale,
										double incrRealP, double opponentIncrRealP,
										int prodScaleRep, int opponentProdScaleRep,
										double incrRepP, double opponentIncrRepP,
										double optCapacity,	double maxCapacity,
										double reductProd, double discount,
										int numberOfTicks, int numberOfRuns, 
										FileWriter verificationOutput, int designPoint
										) throws IOException{
		
		this.resources = resources;
		this.prodScale = prodScale;
		this.opponentProdScale = opponentProdScale;
		this.incrRealP = incrRealP;
		this.opponentIncrRealP = opponentIncrRealP;
		this.prodScaleRep = prodScaleRep;
		this.opponentProdScaleRep = opponentProdScaleRep;
		this.incrRepP = incrRepP;
		this.opponentIncrRepP = opponentIncrRepP;
		this.pMin = pMin;
		this.pMax = MathOperations.roundTo1( pMin + ((prodScale-1) * .1));
		this.optCapacity = optCapacity;
		this.absOptCapacity = (int) ((optCapacity * resources) + .5);	// +.5 for correct rounding (Integer cast cuts decimals)
		this.maxCapacity = maxCapacity;
		this.absMaxCapacity =  (int) ((maxCapacity * resources) + .5); 
		this.reductProd = reductProd;
		this.discount = discount;
		this.numberOfTicks = numberOfTicks;
		this.numberOfRuns = numberOfRuns;
		this.verificationOutput = verificationOutput;
		this.designPoint = designPoint;
		this.delta1 = 0.0;
		this.relDelta1 = 0.0;
		this.relCompensation1= 0.0;
		this.delta2 = 0.0;
		this.relDelta2 = 0.0;
		this.relCompensation2= 0.0;
		this.hits = 0;
		this.loss = 0.0;
		this.generator = new Random ();		
}
	
	/**
	 * Schedules one simulation run
	 * ------------------------------------------------
	 * @throws IOException
	 */
	public void runSimulationExperiment() throws IOException{
		
		for(int index2 = 0; index2 < numberOfRuns; index2++){
		
		for (int index1 = 0; index1 < numberOfTicks; index1++){
			verificationOutput.append("\n" + index1 + ";" + index2);
			if(index1 > 0 && (index1+1) %100 == 0)
				System.out.println("\n Sim.step:" + (index1+1));
			
			// 1 simulation round (tick)
			round();
			
			// observes reporting behavior of division manager agents
			observeReportingBehavior();
			
			// observes equilibrium truthful reporting 
			if(delta1 < 0.001 && delta1 > -0.001 && delta2 < 0.001 && delta2 >-0.001)
				hits++;
			
			// observes division and company success (profit and losses)
			observeSuccess();
			
			// write observations in output file
			printOutput();
			
			relCompensation1 = 0.0;	 
			relCompensation2 = 0.0;
		}
		hits = 0;
		resetRun();
		}
		
		//matrix.close();
	}
	
	/**
	 * Schedules one simulation round (tick).
	 * ----------------------------------------------------------
	 * @throws IOException 
	 */
	public void round() throws IOException {
		/**
		 * Implement simulation round
		 */
	}
	
	
	/**
	 * Reset the agents for the next run
	 * -----------------------------------------------------
	 */
	public void resetRun(){
		/**
		 * Implement simulation round
		 */
	}
	
	/**
	 * Observes profits, losses and payments by the company.
	 * @throws IOException
	 */
	public void observeSuccess() throws IOException{
			/**
			 * Implement observation
			 */
	}
	
	/**
	 * Calculates the relative achieved compensation, in comparison to truthful reporting.
	 * @throws IOException
	 */
	public void relativeCompensation() throws IOException{
			/**
			* Implement relative compensation calculation
			 */
	}
	
	
	/**
	 * Measures truthful reporting as well as under-/overestimation of productivity.
	 */
	public void observeReportingBehavior(){
			/**
			 * Implement observation of reporting behavior
			 */
	}
	
	/**
	 * Company loss comparing to the case that both divisions had reported truthfully.
	 * @return loss
	 * @throws IOException 
	 */
	public double calcCompanyLoss() throws IOException{
	return 	profit - calcTruthfulProfit();
	}
	
	/**
	 * Company profit under truthful reporting by both divisions.
	 * @return truthful profit 
	 * @throws IOException
	 */
	public double calcTruthfulProfit() throws IOException{
		/**
		 * Implement calculation of profit under truthful reporting
		 */
		return 0.0;
	}
	
	/**
	 * Calculates the relative distance between reported and real productivity value, 
	 *  comparing to maximum distance.
	 * @param Index of agent
	 * @return Relative distance
	 */
	public double calcRelDelta(int index){
		/**
		 * Implement calculation.
		 */
		return -1.1;
	}
	
	/**
	 * Creates a random real productivity value.
	 * ------------------------------------------------------------
	 * @return  realValue 
	 * @throws IOException 
	 */
 	public double realProductivity() throws IOException{
 		int random = generator.nextInt(prodScale); 			// nextInt gives a uniform distributed random number
																								// between 0 and n-1
		double realValue = (random * incrRealP) + pMin;
		return MathOperations.roundTo1(realValue);
	}
 	
	/**
	 * Creates a random real productivity value.
	 * ------------------------------------------------------------
	 * @return  realValue 
	 * @throws IOException 
	 */
 	public double realProductivityOpponent() throws IOException{
 		int random = generator.nextInt(opponentProdScale); 			// nextInt gives a uniform distributed random number
																								// between 0 and n-1
		double realValue = (random * opponentIncrRealP) + pMin;
		return MathOperations.roundTo1(realValue);
	}
	
	/**
	 * Creates 1 headquarter and 2 division manager agents
	 * --------------------------------------------------------------------------------
	 * @throws IOException 
	 */
	public void createAgents() throws IOException{
		createDivManagers();
		createHeadquarter();
	}
	
	/**
	 * Initializes learning algorithm of the division manager agent
	 * @throws IOException 
	 */
	public void initializeLA() throws IOException{
		/**
		 * Implement initialization in sub-classes
		 */
	}
	
	/**
	 * Creates 1 headquarter agent
	 */
	public void createHeadquarter(){
		/**
		 * Implement HQ agent
		 */
	}
	
	/**
	 * Creates 2 division manager agents
	 * @throws IOException 
	 */
	public void createDivManagers() throws IOException{
		/**
		 * Create division manager agents
		 */
	}
	
	/**
	 * Print resource matrix
	 * (debugging procedure)
	 * @throws IOException
	 */
	public void printResourceMatrix() throws IOException{
		paymentMatrix.append("Resource Matrix - optC " +  absOptCapacity
									+ " - maxC" +  absMaxCapacity
									+ " - reductProd" + reductProd
									+ "\n;");
		// headline columns
		for (double h = pMin; h < MathOperations.roundTo1(pMin + prodScale * incrRealP); h+=.1){
			paymentMatrix.append(h +";");
		}
		
		for (double i = pMin; i < MathOperations.roundTo1(pMin + prodScale * incrRealP); i+=.1){
			// headline rows
			paymentMatrix.append("\n" + i + ";");
			for (double j = pMin; j < MathOperations.roundTo1(pMin + prodScale * incrRealP); j+=.1){
				// fill matrix
				paymentMatrix.append ( ResourceAllocation.allocate(i, j, resources, this)
											+ "-" + ResourceAllocation.allocate(j, i, resources, this) + ";");
			} 
		}
		
	}
	
	/**
	 * @param realP
	 * @param reportOpponent
	 * @param pMin
	 * @return
	 */
	public double calcMinPayment(double realP,  double reportOpponent) {
		double paymentTmp = 0.0;

		int allocRes1 = ResourceAllocation.allocate(pMin, reportOpponent, resources, this);
		int allocRes2 = ResourceAllocation.allocate(reportOpponent, pMin, resources, this);
		double minPayment =  MathOperations.roundTo1(
																	discount *
																	( 		ProfitCalculation.calcProfit(allocRes1, realP, this)
																			+ 
																			ProfitCalculation.calcProfit(allocRes2, reportOpponent,this)
																	));

		for(double ind = pMin+incrRepP;  ind <= MathOperations.roundTo1( pMin + ((prodScale-1) * .1));  ind =MathOperations.roundTo1(ind +  incrRepP) ){
			allocRes1 = ResourceAllocation.allocate(ind, reportOpponent, resources, this);
			allocRes2 = ResourceAllocation. allocate(reportOpponent, ind, resources, this);
			paymentTmp = 		MathOperations.roundTo1(
																discount * 
																(			ProfitCalculation.calcProfit( allocRes1, realP,  this)
																			+ 
																			ProfitCalculation.calcProfit( allocRes2, reportOpponent,  this)
																));
			
			if(paymentTmp < minPayment)
					minPayment= paymentTmp;
		}
		return minPayment;
}
	public double calcMinPaymentOpp(double realP,  double reportOpponent) {
		double paymentTmp = 0.0;

		int allocRes1 = ResourceAllocation.allocate(pMin, reportOpponent, resources, this);
		int allocRes2 = ResourceAllocation.allocate(reportOpponent, pMin, resources, this);
		double minPayment =  MathOperations.roundTo1(
																	discount *
																	( 		ProfitCalculation.calcProfit(allocRes1, realP, this)
																			+ 
																			ProfitCalculation.calcProfit(allocRes2, reportOpponent,this)
																	));

		for(double ind = pMin+opponentIncrRepP;  ind <= MathOperations.roundTo1( pMin + ((opponentProdScale-1) * .1));  ind =MathOperations.roundTo1(ind +  opponentIncrRepP) ){
			allocRes1 = ResourceAllocation.allocate(ind, reportOpponent, resources, this);
			allocRes2 = ResourceAllocation. allocate(reportOpponent, ind, resources, this);
			paymentTmp = 		MathOperations.roundTo1(
																discount * 
																(			ProfitCalculation.calcProfit( allocRes1, realP,  this)
																			+ 
																			ProfitCalculation.calcProfit( allocRes2, reportOpponent,  this)
																));
			
			if(paymentTmp < minPayment)
					minPayment= paymentTmp;
		}
		return minPayment;
	}
	
	/**
	 * Writes observations in output file.
	 * (for simulation data analysis)
	 * @throws IOException
	 */
	public void printOutput() throws IOException{
		/**
		 * Implement output in csv file.
		 */
	}
	
	/**
	 * Prints setting for simulation run.
	 * (debugging procedure)
	 */
	public void printSetting(){
		System.out.println(new String(
				"\nResources " + resources +
				"\nprodScale " + prodScale +
				"\npMin " + pMin +
				"\npMax " + pMax +
				"\noptCapacity " + absOptCapacity+
				"\nmaxCapacity " +absMaxCapacity+
				"\nreductProd " + reductProd+
				"\ndiscount" + discount +
				"\nnumberOfTicks " + numberOfTicks+
				"\nnumberOfRuns " + numberOfRuns
				));		
	}
	
	public void print(String text){
		System.out.println(text);
	}

	/**
	 * Checks setting for valid inputs
	 */
	public void checkSetting(){
		
		if (absMaxCapacity < absOptCapacity  ){
			JOptionPane.showMessageDialog(f,
				    "Invalid input: The maximum capacity value has to be higher than optimal capacity. Please try again.",
				    "LAMDA - error message",
				    JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
		
		if (reductProd > 1 || reductProd < 0 ){
			JOptionPane.showMessageDialog(f,
				    "Invalid input: The reduction for productivity between max and opt capacity has to be in interval [0,1]. Please try again.",
				    "LAMDA - error message",
				    JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
		
		if (discount > 1 || discount < 0 ){
			JOptionPane.showMessageDialog(f,
				    "Invalid input: The discount factor has to be in interval [0,1]. Please try again.",
				    "LAMDA - error message",
				    JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}		
	}
	
	public FileWriter getVerificationOutput() {
		return verificationOutput;
	}

	public int getResources() {
		return resources;
	}

	public int getProdScale() {
		return prodScale;
	}

	public int getNumberOfTicks() {
		return numberOfTicks;
	}

	public int getNumberOfRuns() {
		return numberOfRuns;
	}

	public int getOptCapacity() {
		return absOptCapacity;
	}

	public int getMaxCapacity() {
		return absMaxCapacity;
	}

	public double getReductProd() {
		return reductProd;
	}
}
